@@ -1,8 +1,8 @@
 import logging
+import pytz
 from dataclasses import dataclass
 from datetime import datetime, timedelta
 from abc import ABC, abstractmethod
-from typing import Optional
 from sklearn import svm
 from typing import List
 from pvpower.weather_forecast import WeatherForecast
@@ -16,7 +16,8 @@ class Vectorizer(ABC):
         self._datetime_resolution_minutes = datetime_resolution_minutes
 
     def _minutes_of_day(self, dt: datetime) -> int:
-        return (dt.hour * 60) + dt.minute
+        utc_time = dt.astimezone(pytz.UTC)
+        return (utc_time.hour * 60) + utc_time.minute
 
     def _scale(self, value: int, max_value: int, digits=1) -> float:
         if value == 0:
@@ -65,41 +66,36 @@ class Estimator:
         else:
             self.__vectorizer = vectorizer
         self.num_samples_last_train = 0
+        # initialize with dummy data
+        self.retrain([LabelledWeatherForecast(datetime.now(), 1, 0, 0, 0, 0, 0), LabelledWeatherForecast(datetime.now() - timedelta(days=1), 1, 0, 0, 0, 0, 1)])
+
+    def usable_as_train_sample(self, sample: LabelledWeatherForecast) -> bool:
+        return sample.irradiance > 0 \
+               and sample.sunshine is not None \
+               and sample.visibility is not None
 
     def clean_data(self, samples: List[LabelledWeatherForecast]) -> List[LabelledWeatherForecast]:
         seen = list()
         samples = list(filter(lambda sample: seen.append(sample.time) is None if sample.time not in seen else False, samples))
-        samples = [sample for sample in samples if sample.irradiance > 0]
-        samples = [sample for sample in samples if sample.sunshine is not None and sample.visibility is not None]
+        samples = [sample for sample in samples if self.usable_as_train_sample(sample)]
         return samples
 
     def retrain(self, samples: List[LabelledWeatherForecast]) -> TrainReport:
         cleaned_samples = self.clean_data(samples)
-        if len(cleaned_samples) < 2:
-            # dummy values
-            logging.info("no train available. Using dummy values")
-            cleaned_samples = [LabelledWeatherForecast(datetime.now(), 0, 0, 0, 0, 0, 0),
-                               LabelledWeatherForecast(datetime.now() - timedelta(days=1), 0, 0, 0, 0, 0, 0)]
-        num_samples = len(cleaned_samples)
-        if self.num_samples_last_train != num_samples:
-            feature_vector_list = [self.__vectorizer.vectorize(sample) for sample in cleaned_samples]
-            label_list = [sample.power_watt for sample in cleaned_samples]
-            if len(set(label_list)) > 1:
-                self.__clf.fit(feature_vector_list, label_list)
-                self.__num_samples_last_train = num_samples
+        feature_vector_list = [self.__vectorizer.vectorize(sample) for sample in cleaned_samples]
+        label_list = [sample.power_watt for sample in cleaned_samples]
+        if len(set(label_list)) > 1:
+            self.__clf.fit(feature_vector_list, label_list)
+            self.num_samples_last_train = len(cleaned_samples)
         return TrainReport(cleaned_samples)
 
     def __str__(self):
         return "Model vectorizer=" + str(self.__vectorizer) + " trained with " + str(self.num_samples_last_train) + " cleaned samples"
 
-    def predict(self, sample: WeatherForecast) -> Optional[int]:
-        try:
-            if sample.irradiance > 0:
-                feature_vector = self.__vectorizer.vectorize(sample)
-                predicted = self.__clf.predict([feature_vector])[0]
-                return int(predicted)
-            else:
-                return 0
-        except Exception as e:
-            logging.warning("error occurred predicting " + str(sample), e)
-            return None
+    def predict(self, sample: WeatherForecast) -> int:
+        if sample.irradiance > 0:
+            feature_vector = self.__vectorizer.vectorize(sample)
+            predicted = self.__clf.predict([feature_vector])[0]
+            return int(predicted)
+        else:
+            return 0
